import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class Providers(RestController):
    """
    MarcusLion Providers class
    $ curl 'https://qa1.marcuslion.com/core/datasets/providers'
    """

    def __init__(self):
        super().__init__(api_version + "/datasets/providers")

    def list(self) -> pd.DataFrame:
        """
        Providers.list()
        """
        return super().verify_get("", {})

    # def search(self, search) -> pd.DataFrame:
    #     """
    #     Providers.search(search)
    #     """
    #     params = {"search": search}
    #     return super().verify_get("search",  params)

    def query(self, uid) -> pd.DataFrame:
        """
        Providers.query()
        """
        params = {"id": uid}
        return super().verify_get("",  params)





import io
import urllib
import urllib3

import pandas

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class TimeSeries(RestController):
    """
    https://qa1.marcuslion.com/swagger-ui/index.html#/time-series-api-controller
    """

    def __init__(self):
        super().__init__(api_version + "/timeseries/")

    def list(self) -> pandas.DataFrame:
        """
        Indicators.list()
        """
        return super().verify_get()

    def list(self, symbol, interval, page_size) -> pandas.DataFrame:
        """
        Indicators.list()
        """
        return super().verify_get(symbol + "/" + interval + "/" + str(page_size), {})

    def query(self, ref):
        return super().verify_get_data("query", {"ref", ref})

    def search(self, search) -> pandas.DataFrame:
        return super().verify_get_data("search", {"search", search})

    def candles(self, params) -> pandas.DataFrame:
        """
        TimeSeries.candles(ref, params)
        """
        res = super().verify_get("history", params)
        if res is None or 'data' not in res:
            return pandas.DataFrame()

        df = pandas.DataFrame(res['data'], columns=res['schema'])
        df.name = res['entityKey']['symbol'] + "_" + res['interval']
        df["timestamp"] = pandas.to_datetime(df["timestamp"], unit='ms')
        return df

    def trades(self, params) -> pandas.DataFrame:
        """
        TimeSeries.trades(ref, params)
        """
        res = super().verify_get("history/raw", params)
        if res is None or 'data' not in res:
            return pandas.DataFrame()

        df = pandas.DataFrame(res['data'], columns=res['schema'])
        df.name = res['entityKey']['symbol'] + "_trades"
        df["timestamp"] = pandas.to_datetime(df["timestamp"], unit='ms')
        return df

    def subscribe(self, ref, params):
        """
        TimeSeries.subscribe(ref, params)
        """
        pass



import json
import pandas as pd
import urllib3

from marcuslion.config import base_url, api_key
from marcuslion.apiUtils import get_filename_from_api


class RestController:
    """
    MarcusLion RestController class
    """
    _http = urllib3.PoolManager()

    def __init__(self, url):
        self.url = base_url

        if not self.url.endswith('/'):
            self.url += '/'

        self.url += url

    def _prepare_params(self, action):
        s = self.url
        if not action:
            return s
        if not action.startswith('/') and not s.endswith('/'):
            s += '/'
        s += action
        return s

    def _get_request(self, action, params, method="GET", preload_content=True, **kwargs):
        full_url = self._prepare_params(action)

        print(f"_get_request {full_url} {params}")

        resp = self._http.request(method, full_url,
                                  headers={'X-MARCUSLION-API-KEY': api_key},
                                  fields=params,
                                  preload_content=preload_content,
                                  **kwargs)

        if resp.status == 401:
            raise ValueError("401: Unauthorized User. URL:" + full_url)
        if resp.status != 200:
            raise ValueError("status: " + full_url + " -> " + (
                str(resp.status) + (" data: " + resp.data.decode()) if resp.data else ""))
        return resp

    def download_file(self, action, params, output_path=None):
        if output_path is None:
            output_path = "."
        resp = self._get_request(action, params, preload_content=False)
        file_name = f"{output_path}/{get_filename_from_api(resp)}"
        with open(file_name, 'wb') as out_file:
            for chunk in resp.stream(1024):
                out_file.write(chunk)

        return out_file.name

    def verify_get_df(self, action=None, params=None) -> pd.DataFrame:
        response = self.verify_get(action, params)
        return pd.DataFrame.from_records(response)

    def verify_get(self, action=None, params=None) -> any:
        # Sending a GET request and getting back response as HTTPResponse object.
        resp = self._get_request(action, params)

        data_str = resp.data.decode()
        if len(data_str) == 0:
            return None

        return json.loads(data_str)

    def verify_get_data(self, action=None, params=None) -> pd.DataFrame:
        data = self.verify_get(action, params)
        if data is None:
            return pd.DataFrame()
        df = pd.DataFrame(data['data'])
        return df



def get_filename_from_api(response, default_filename="defaultFie"):
    content_disposition = response.headers.get('Content-Disposition')
    if content_disposition:
        return content_disposition.split("filename=")[1].split(";")[0].replace("\"", "")
    return default_filename



import logging
import ssl
import skimage as ski
import os.path
import pandas as pd

from marcuslion.datasets import Datasets
from marcuslion.projects import Projects
from marcuslion.providers import Providers
from marcuslion.dataproviders import DataProviders
from marcuslion.documents import Documents
from marcuslion.indicators import Indicators
from marcuslion.timeseries import TimeSeries
from marcuslion.models import Models
from marcuslion.dataframes import DataFrames
from marcuslion.support import Support
from marcuslion.ussec import UsSec

providers = Providers()
data_providers = DataProviders()
datasets = Datasets()
dataframes = DataFrames()
documents = Documents()
indicators = Indicators()
timeseries = TimeSeries()
models = Models()
support = Support()
projects = Projects(datasets)
us_sec = UsSec()


logger = logging.getLogger()
# logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())

logger.info("Start marcuslion lib")

pd.set_option('display.max_rows', 500)
pd.set_option('display.max_columns', 500)
pd.set_option('display.width', 2000)

# resource_path = os.path.join(os.path.split(__file__)[0], "resources")
# image = ski.io.imread(resource_path + "/marcus_lion_logo.jpg")
# ski.io.imshow(image)

# this required downgrading requests library to urllib3-1.24.3 to avoid SSL cert error
ssl._create_default_https_context = ssl._create_unverified_context

print("MarcusLion lib loaded")


def help1():
    print("  ml.providers.list() or ml.datasets.search()")








import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class Indicators(RestController):
    """
    MarcusLion Indicators class
    """

    def __init__(self):
        super().__init__(api_version + "/indicators")

    def list(self) -> pd.DataFrame:
        """
        Indicators.list()
        """
        return super().verify_get_df("", {})

    def query(self, uuid):
        return super().verify_get(uuid)

    def search(self, search) -> pd.DataFrame:
        data = super().verify_get("search", {"query": search})  # {"query", search}
        return data

    def download(self, params) -> pd.DataFrame:
        """
        Indicators.download(params)
        """
        res = super().verify_get("download", params)
        if res is None or 'indicator' not in res or 'data' not in res['indicator']:
            return pd.DataFrame()
        return pd.DataFrame(res['indicator']['data'], columns=res['indicator']['schema'])

    def subscribe(self, params) -> pd.DataFrame:
        """
        Indicators.subscribe(params)
        """
        return super().verify_get("subscribe", params)

    def recently(self):
        """
        Indicators.recently
        """
        data = super().verify_get("recently")
        return data



import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class Datasets(RestController):
    """
    MarcusLion Datasets class
        # $ curl 'https://qa1.marcuslion.com/core/datasets/search?providers=kaggle,usgov&title=bike'
    """

    def __init__(self):
        super().__init__(api_version + "/datasets")

    def list(self) -> pd.DataFrame:
        """
        Datasets.list()
        """
        return super().verify_get_data("", {})

    def search(self, search, provider_list) -> pd.DataFrame:
        """
        Datasets.search()
        """
        params = {"providers": provider_list, "title": search}

        return super().verify_get_data("search", params)

    def query(self, title) -> pd.DataFrame:
        """
        Datasets.query(ref)
        """
        return super().verify_get_data("search", {"title": title})

    def download(self, source, ref, file_name, output_path=None) -> any:
        """
        Datasets.download(ref)
        """
        print(f"Downloading {file_name} from {source}/{ref}...")
        return super().download_file(source + "/download", {"ref": ref, "file": file_name}, output_path)
